@@ -1,6 +1,6 @@
 package Net::SFTP::Foreign::Backend::Windows;
 
-our $VERSION = '0.01';
+our $VERSION = '1.63_05';
 
 use strict;
 use warnings;
@@ -20,11 +20,44 @@ sub _defaults {
 }
 
 sub _init_transport_streams {
-    my ($self, $sftp) = @_;
+    my ($backend, $sftp) = @_;
     binmode $sftp->{ssh_in};
     binmode $sftp->{ssh_out};
 }
 
+sub _open_dev_null {
+    my $sftp = shift;
+    my $dev_null;
+    unless (open $dev_null, '>', 'NUL:') {
+	$sftp->_conn_failed("Unable to redirect stderr for slave SSH process to NUL: $!");
+	return;
+    }
+    $dev_null
+}
+
+# workaround for IPC::Open3 not working with tied filehandles even
+# when they implement FILENO
+sub _open3 {
+    my $backend = shift;
+    my $sftp = shift;
+    if (tied(*STDERR)) {
+	my $fn = eval { defined $_[2] ? fileno $_[2] : fileno *STDERR };
+	unless (defined $fn and $fn >= 0) {
+	    $sftp->_conn_failed("STDERR or stderr_fh is not a real file handle: " . (length $@ ? $@ : $!));
+	    return;
+	}
+	local *STDERR;
+	unless (open STDERR, ">&=$fn") {
+	    $sftp->_conn_failed("Unable to reattach STDERR to fd $fn: $!");
+	    return;
+	}
+	$backend->SUPER::_open3($sftp, @_);
+    }
+    else {
+	$backend->SUPER::_open3($sftp, @_);
+    }
+}
+
 sub _sysreadn {
     my ($sftp, $n) = @_;
     my $bin = \$sftp->{_bin};
@@ -41,7 +74,7 @@ sub _sysreadn {
 }
 
 sub _do_io {
-    my ($self, $sftp, $timeout) = @_;
+    my ($backend, $sftp, $timeout) = @_;
 
     return undef unless $sftp->{_connected};
 
@@ -57,14 +90,16 @@ sub _do_io {
 	substr($$bout, 0, $written, "");
     }
 
+    defined $timeout and $timeout <= 0 and return;
+
     _sysreadn($sftp, 4) or return undef;
 
     my $len = 4 + unpack N => $$bin;
     if ($len > 256 * 1024) {
-	$sftp->_set_status(SSH2_FX_BAD_MESSAGE);
-	$sftp->_set_error(SFTP_ERR_REMOTE_BAD_MESSAGE,
-			  "bad remote message received");
-	return undef;
+        $sftp->_set_status(SSH2_FX_BAD_MESSAGE);
+        $sftp->_set_error(SFTP_ERR_REMOTE_BAD_MESSAGE,
+                          "bad remote message received");
+        return undef;
     }
     _sysreadn($sftp, $len);
 }
